@@ -1,28 +1,25 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 """
-RDFLib Python binding for OWL Abstract Syntax
-=============================================
+Overview of InfixOWL
+--------------------
 
-copied from:
+The core OWL abstract syntax is implemented based on this recipe:
 
-    * http://code.google.com/p/fuxi/source/browse/lib/Syntax/InfixOWL.py
-see:
+    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/384122
 
-    * http://www.w3.org/TR/owl-semantics/syntax.html
-    * http://owl-workshop.man.ac.uk/acceptedLong/submission_9.pdf
+Quoting the original InfixOWL docstrings,
 
-3.2.3 Axioms for complete classes without using owl:equivalentClass
--------------------------------------------------------------------
+    Python has the wonderful "in" operator and it would be nice
+    to have additional infix operator like this. This recipe shows
+    how (almost) arbitrary infix operators can be defined.
 
-Named class description of type 2 (with owl:oneOf) or type 4-6 (with owl:intersectionOf, owl:unionOf or owl:complementOf
-  
-Uses Manchester Syntax for __repr__  
------------------------------------
+Some usage examples follow below.
 
->>> g = Graph()    
+>>> from ordf.namespace import register_ns
 >>> exNs = Namespace("http://example.com/")
->>> bind_ns(g, { "ex": exNs })
+>>> register_ns("ex", exNs)
+>>> g = Graph()
 
 Now we have an empty graph, we can construct OWL classes in it
 using the Python classes defined in this module
@@ -43,11 +40,14 @@ We can then access the rdfs:subClassOf relationships
 
 This can also be used against already populated graphs:
 
-#>>> owlGraph = Graph().parse(OWL)
-#>>> namespace_manager.bind('owl', OWL, override=False)
-#>>> owlGraph.namespace_manager = namespace_manager
-#>>> list(Class(OWL.Class,graph=owlGraph).subClassOf)
-#[Class: rdfs:Class ]
+.. code-block:: python
+
+    owlGraph = Graph().parse(OWL)
+    namespace_manager.bind('owl', OWL, override=False)
+    owlGraph.namespace_manager = namespace_manager
+    list(Class(OWL.Class,graph=owlGraph).subClassOf)
+
+    --> [Class: rdfs:Class ]
 
 Operators are also available.  For instance we can add ex:Opera to the extension
 of the ex:CreativeWork class via the '+=' operator
@@ -56,7 +56,7 @@ of the ex:CreativeWork class via the '+=' operator
 Class: ex:Opera SubClassOf: ex:MusicalWork
 >>> b = Class(exNs.CreativeWork,graph=g)
 >>> b += a
->>> print list(a.subClassOf)
+>>> list(a.subClassOf)
 [Class: ex:CreativeWork , Class: ex:MusicalWork ]
 
 And we can then remove it from the extension as well
@@ -100,14 +100,34 @@ owl:Restrictions can also be instanciated:
 ( ex:hasParent only ex:Human )
 
 Restrictions can also be created using Manchester OWL syntax in 'colloquial' Python 
+
 >>> exNs.hasParent |some| Class(exNs.Physician,graph=g)
 ( ex:hasParent some ex:Physician )
-
 >>> Property(exNs.hasParent,graph=g) |max| Literal(1)
 ( ex:hasParent max 1 )
 
-#>>> print g.serialize(format='pretty-xml')
+Datatype and Object Properties
+------------------------------
 
+.. autoclass:: predicate
+    :show-inheritance:
+.. autoclass:: object_predicate
+    :show-inheritance:
+.. autoclass:: cached_predicate
+    :show-inheritance:
+
+Core classes
+------------
+
+.. autoclass:: Individual
+.. autoclass:: AnnotatibleTerms
+    :show-inheritance:
+.. autoclass:: Ontology
+    :show-inheritance:
+.. autoclass:: Class
+    :show-inheritance:
+.. autoclass:: Property
+    :show-inheritance:
 """
 import os, itertools
 from pprint import pprint
@@ -115,19 +135,10 @@ from pprint import pprint
 from ordf.collection import Collection
 from ordf.graph import Graph
 from ordf.namespace import bind_ns, Namespace, OWL, RDF, RDFS, XSD
-from ordf.term import BNode, Identifier, Literal, URIRef, Variable
+from ordf.term import Node, BNode, Identifier, Literal, URIRef, Variable
 
 from rdflib.util import first
 
-"""
-From: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/384122
-
-Python has the wonderful "in" operator and it would be nice to have additional 
-infix operator like this. This recipe shows how (almost) arbitrary infix 
-operators can be defined.
-
-"""
-
 ##
 ## Data fixtures for ORDF
 ##
@@ -155,6 +166,177 @@ def inference_rules(handler, network):
     dlp = network.setupDescriptionLogicProgramming(owl, addPDSemantics=True)
     return list(rules) + dlp
 
+# attribute properties
+class predicate(property):
+    """
+    This function provides facilities that look a bit like the properties
+    on a `Django`_ or `SQLAlchemy`_ class.
+
+    .. _Django: http://www.djangoproject.com/
+    .. _SQLAlchemy: http://www.sqlalchemy.org/
+
+    It is used when you make a subclass of :class:`Individual` or one
+    of its subclasses so that you can put properties on. Here is an 
+    example taken from the :mod:`ordf.vocab.foaf` module:
+
+    >>> from ordf.namespace import FOAF
+    >>> class Person(Class):
+    ...     name = predicate(FOAF.name)
+    ...
+    >>> norman = Person()
+    >>> norman.name = Literal("Norman Bethune")
+    >>> [str(x) for x in norman.name]
+    ['Norman Bethune']
+    >>> del norman.name
+    >>> len(list(norman.name))
+    0
+
+    The *set()* operation can take either a literal or a list/tuple
+    of literals.
+    """
+    def __init__(self, term):
+        self.term = term
+        super(predicate, self).__init__(self._get, self._set, self._del)
+    def _get(self, obj):
+        return obj.graph.distinct_objects(obj.identifier, self.term)
+    def _set(self, obj, value):
+        if not value: return
+        if not isinstance(value, list) and not isinstance(value, tuple):
+            value = (value,)
+        for x in value:
+            if not isinstance(x, Node) and not isinstance(x, Individual):
+                x = Literal(x)
+            obj.graph.add((obj.identifier, self.term, classOrTerm(x)))
+    def _del(self, obj):
+        obj.graph.remove((obj.identifier, self.term, None))
+
+class object_predicate(predicate):
+    """
+    This function behaves similarly to :class:`predicate` except that
+    rather than returning the term associated with the predicate it
+    will attempt to return an instantiated Individual (or subclass
+    thereof). In this way such a predicate will behave somewhat like
+    a foreign key relationship as implemented in SQL ORMs.
+
+    >>> from ordf.namespace import Namespace
+    >>> EX = Namespace("http://example.org/")
+    >>> class Country(Class):
+    ...     pass
+    ...
+    >>> class Province(Class):
+    ...     country = object_predicate(EX.country, Country)
+    ...
+    >>> canada = Country(EX.canada)
+    >>> canada.label = "Canada"
+    >>> quebec = Province(EX.quebec, graph=canada.graph)
+    >>> quebec.label = "Québec"
+    >>> quebec.country = canada
+    >>> for country in quebec.country: print [str(x) for x in country.label]
+    ['Canada']
+    """
+    def __init__(self, term, implementation=None):
+        if implementation is None:
+            implementation = Individual
+        assert issubclass(implementation, Individual), "expected an Individual, got an %s" % implementation
+        super(object_predicate, self).__init__(term)
+        self._cls = implementation
+    def _get(self, obj):
+        for identifier in super(object_predicate, self)._get(obj):
+            yield self._cls(identifier, graph=obj.graph)
+
+class cached_predicate(predicate):
+    """
+    This version of the :class:`predicate` function behaves the
+    same way except that with *get()* operations the returned values
+    are cached. Doing a *set()* or *del()* operation causes the cache
+    to be cleared. Usage is essentially identical.
+
+    >>> from ordf.namespace import DC
+    >>> class Book(Class):
+    ...    title = cached_predicate(DC.subject)
+    ...
+    >>> we = Book()
+    >>> we.title = (Literal("We", lang="en"), Literal("Nous", lang="fr"))
+    >>> [str(x) for x in we.title]
+    ['We', 'Nous']
+    >>> ### prove that the value is cached
+    >>> we.graph.remove((we.identifier, None, None))
+    >>> len(list(we.title))
+    2
+    >>> del we.title
+    >>> len(list(we.title))
+    0
+    """
+    def _cache(self, obj):
+        if not hasattr(obj, "__predicate_cache__"):
+            obj.__predicate_cache__ = {}
+        return obj.__predicate_cache__
+    def _get(self, obj):
+        _cache = self._cache(obj)
+        if self.term in _cache:
+            return _cache[self.term]
+        results = list(super(cached_predicate, self)._get(obj))
+        _cache[self.term] = results
+        return results
+    def _set(self, obj, value):
+        super(cached_predicate, self)._set(obj, value)
+        _cache = self._cache(obj)
+        if self.term in _cache: del _cache[self.term]
+    def _del(self, obj):
+        super(cached_predicate, self)._del(obj)
+        _cache = self._cache(obj)
+        if self.term in _cache: del _cache[self.term]
+
+class cached_object_predicate(cached_predicate):
+    """
+    And the caching analogue to :class:`object_predicate`.
+
+    >>> from ordf.namespace import Namespace
+    >>> EX = Namespace("http://example.org/")
+    >>> class Cat(Class):
+    ...     pass
+    >>> class Person(Class):
+    ...     pet = cached_object_predicate(EX.pet, Cat)
+    ...
+    >>> bob = Person(EX.bob)
+    >>> bob.label = "Bob"
+    >>> max = Cat(EX.max, graph=bob.graph)
+    >>> max.label = "Max"
+    >>> bob.pet = max
+    >>> for pet in bob.pet: print [str(x) for x in pet.label]
+    ['Max']
+    >>> # prove the data has been cached
+    >>> bob.graph.remove((bob.identifier, None, None))
+    >>> for pet in bob.pet: print [str(x) for x in pet.label]
+    ['Max']
+    """
+    def __init__(self, term, implementation=None):
+        if implementation is None:
+            implementation = Individual
+        assert issubclass(implementation, Individual), "expected an Individual, got an %s" % implementation
+        super(cached_object_predicate, self).__init__(term)
+        self._cls = implementation
+    def _set(self, obj, value):
+        if not isinstance(value, list) and not isinstance(value, tuple):
+            value = [value]
+        for x in value:
+            if isinstance(x, URIRef) or isinstance(x, BNode):
+                x = self._cls(value, graph=obj.graph)
+            assert isinstance(x, Individual), "expected an Individual, got an %s" % x
+            obj.graph.add((obj.identifier, self.term, x.identifier))
+        _cache = self._cache(obj)
+        if self.term in _cache: del _cache[self.term]
+    def _get(self, obj):
+        _cache = self._cache(obj)
+        if self.term in _cache:
+            return _cache[self.term]
+        results = [
+            self._cls(x, graph=obj.graph)
+            for x in super(cached_object_predicate, self)._get(obj)
+            ]
+        _cache[self.term] = results
+        return results
+
 # definition of an Infix operator class
 # this recipe also works in jython
 # calling sequence for the infix is either:
@@ -162,7 +344,7 @@ def inference_rules(handler, network):
 # or:
 # x <<op>> y
 
-class Infix:
+class Infix(object):
     def __init__(self, function):
         self.function = function
     def __ror__(self, other):
@@ -302,13 +484,52 @@ class TermDeletionHelper:
 
 class Individual(object):
     """
-    A typed individual
+    A typed individual, the base class of the InfixOWL classes.
+    
+    :param identifier:
+        The identifier for this individual, a :class:`ordf.term.URIRef`
+
+    :param graph:
+        The graph which will provide the backing store for the individual.
+        If *None* a new graph will be created.
+
+    .. attribute:: identifier
+
+        The identifier attribute is handled specially to make it 
+        immutable.
+
+    .. attribute:: graph
+
+        The graph acting as a backing store is always accessible via
+        this attribute on instances of :class:`Individual`
+
+    .. attribute:: factoryGraph
+
+        The usual practice is to set this equal to the :attr:`graph` 
+        attribute, it is the source of data used by :meth:`serialize`
+
+    .. attribute:: type
+
+        Type is essentially the result of using the :func:`predicate` 
+        function with *rdf:type*
+
+    .. attribute:: cached_types
+
+        A cached version of the :attr:`type` attribute.
+
+    .. attribute:: sameAs
+
+        Conceptually similar with *owl:sameAs* but special handling is
+        required since it will return a full instance of a subclass of
+        :class:`Individual`.
+
+    .. automethod:: clearInDegree
+    .. automethod:: clearOutDegree
+    .. automethod:: delete
+    .. automethod:: replace
+    .. automethod:: serialize
     """
-    factoryGraph = Graph()
-    def serialize(self,graph):
-        for fact in self.factoryGraph.triples((self.identifier,None,None)):
-            graph.add(fact)
-    def __init__(self, identifier=None,graph=None):
+    def __init__(self, identifier=None, graph=None):
         self.__identifier = identifier is not None and identifier or BNode()
         if graph is None:
             self.graph = self.factoryGraph
@@ -322,50 +543,49 @@ class Individual(object):
             except:
                 pass
     
+    factoryGraph = Graph()
+    def serialize(self, graph):
+        """
+        Take terms related to this individual using a blank node
+        closure and add them to the provided graph.
+        """
+        graph += self.factoryGraph.bnc((self.identifier,None,None))
+
     def clearInDegree(self):
+        """
+        Remove references to this individual as an object in the
+        backing store.
+        """
         self.graph.remove((None,None,self.identifier))    
 
     def clearOutDegree(self):
+        """
+        Remove all statements to this individual as a subject in the
+        backing store. Note that this only removes the statements 
+        themselves, not the blank node closure so there is a chance
+        that this will cause orphaned blank nodes to remain in the 
+        graph.
+        """
         self.graph.remove((self.identifier,None,None))    
 
     def delete(self):
+        """
+        Delete the individual from the graph, clearing the in and
+        out degrees.
+        """
         self.clearInDegree()
         self.clearOutDegree()
         
     def replace(self,other):
+        """
+        Replace the individual in the graph with the given other,
+        causing all triples that refer to it to be changed and then
+        delete the individual.
+        """
         for s,p,o in self.graph.triples((None,None,self.identifier)):
             self.graph.add((s,p,classOrIdentifier(other)))
         self.delete()
     
-    def _get_type(self):
-        for _t in self.graph.objects(subject=self.identifier,predicate=RDF.type):
-            yield _t
-    def _set_type(self, kind):
-        if not kind:
-            return  
-        if isinstance(kind,(Individual,Identifier)):
-            self.graph.add((self.identifier,RDF.type,classOrIdentifier(kind)))
-        else:
-            for c in kind:
-                assert isinstance(c,(Individual,Identifier))
-                self.graph.add((self.identifier,RDF.type,classOrIdentifier(c)))
-                
-    @TermDeletionHelper(RDF.type)            
-    def _delete_type(self):
-        """
-        >>> g = Graph()
-        >>> b=Individual(OWL.Restriction,g)
-        >>> b.type = RDF.Resource
-        >>> len(list(b.type))
-        1
-        >>> del b.type
-        >>> len(list(b.type))
-        0
-        """
-        pass
-                
-    type = property(_get_type, _set_type, _delete_type)
-
     def _get_identifier(self):
         return self.__identifier
     def _set_identifier(self, i):
@@ -388,119 +608,60 @@ class Individual(object):
                 pass
             
     identifier = property(_get_identifier, _set_identifier)
+    type = predicate(RDF.type)
+    cached_types = cached_predicate(RDF.type)
+    sameAs = predicate(OWL.sameAs)
     
-    def _get_sameAs(self):
-        for _t in self.graph.objects(subject=self.identifier,predicate=OWL.sameAs):
-            yield _t
-    def _set_sameAs(self, term):
-        if not kind:
-            return  
-        if isinstance(term,(Individual,Identifier)):
-            self.graph.add((self.identifier,OWL.sameAs,classOrIdentifier(term)))
-        else:
-            for c in term:
-                assert isinstance(c,(Individual,Identifier))
-                self.graph.add((self.identifier,OWL.sameAs,classOrIdentifier(c)))
-                
-    @TermDeletionHelper(OWL.sameAs)            
-    def _delete_sameAs(self):
-        pass
-                
-    sameAs = property(_get_sameAs, _set_sameAs, _delete_sameAs)
-    
-
 class AnnotatibleTerms(Individual):
     """
     Terms in an OWL ontology with rdfs:label and rdfs:comment
+
+    .. attribute:: isDefinedBy
+
+        The result of calling the :func:`predicate` function on
+        *rdfs:isDefinedBy*
+
+    .. attribute:: label
+
+        Similarly for *rdfs:label*
+
+    .. attribute:: comment
+
+        Similarly for *rdfs:comment*
+
+    .. attribute:: seeAlso
+
+        Similarly for *rdfs:seeAlso*
     """
     def __init__(self,identifier,graph=None):
         super(AnnotatibleTerms, self).__init__(identifier,graph)
-    def _get_comment(self):
-        for comment in self.graph.objects(subject=self.identifier,predicate=RDFS.comment):
-            yield comment
-    def _set_comment(self, comment):
-        if not comment:
-            return        
-        if isinstance(comment,Identifier):
-            self.graph.add((self.identifier,RDFS.comment,comment))
-        else:
-            for c in comment:
-                self.graph.add((self.identifier,RDFS.comment,c))
-                
-    @TermDeletionHelper(RDFS.comment)
-    def _del_comment(self):
-        pass
-    
-    comment = property(_get_comment, _set_comment, _del_comment)
 
-    def _get_seeAlso(self):
-        for sA in self.graph.objects(subject=self.identifier,predicate=RDFS.seeAlso):
-            yield sA
-    def _set_seeAlso(self, seeAlsos):
-        if not seeAlsos:
-            return        
-        for s in seeAlsos:
-            self.graph.add((self.identifier,RDFS.seeAlso,s))
-            
-    @TermDeletionHelper(RDFS.seeAlso)
-    def _del_seeAlso(self):
-        pass
-    seeAlso = property(_get_seeAlso, _set_seeAlso, _del_seeAlso)
+    label = predicate(RDFS.label)
+    comment = predicate(RDFS.comment)
+    seeAlso = predicate(RDFS.seeAlso)
+    isDefinedBy = predicate(RDFS.isDefinedBy)
 
-    def _get_label(self):
-        for label in self.graph.objects(subject=self.identifier,predicate=RDFS.label):
-            yield label
-    def _set_label(self, label):
-        if not label:
-            return        
-        if isinstance(label,Identifier):
-            self.graph.add((self.identifier,RDFS.label,label))
-        else:
-            for l in label:
-                self.graph.add((self.identifier,RDFS.label,l))
-                
-    @TermDeletionHelper(RDFS.label)
-    def _delete_label(self):
-        """
-        >>> g=Graph()
-        >>> b=Individual(OWL.Restriction,g)
-        >>> b.label = Literal('boo')
-        >>> len(list(b.label))
-        1
-        >>> del b.label
-        >>> len(list(b.label))
-        0
-        """
-        pass
-        
-    label = property(_get_label, _set_label, _delete_label)
 
 class Ontology(AnnotatibleTerms):
-    """ The owl ontology metadata"""
+    """
+    The owl ontology metadata
+
+    .. attribute:: versionInfo
+
+        The result of calling :func:`predicate` on *owl:versionInfo*
+
+    .. attribute:: imports
+
+        Ditto with *owl:imports*
+    """
     def __init__(self, identifier=None,imports=None,comment=None,graph=None):
         super(Ontology, self).__init__(identifier,graph)
         self.imports = imports and imports or []
         self.comment = comment and comment or []
         if (self.identifier,RDF.type,OWL.Ontology) not in self.graph:
-            self.graph.add((self.identifier,RDF.type,OWL.Ontology))
-
-    def setVersion(self,version):
-        self.graph.set((self.identifier,OWL.versionInfo,version))
-
-    def _get_imports(self):
-        for owl in self.graph.objects(subject=self.identifier,predicate=OWL['imports']):
-            yield owl
-    def _set_imports(self, other):
-        if not other:
-            return        
-        for o in other:
-            self.graph.add((self.identifier,OWL['imports'],o))
-            
-    @TermDeletionHelper(OWL['imports'])
-    def _del_imports(self):            
-        pass
-    
-    imports = property(_get_imports, _set_imports, _del_imports)
+            self.type = OWL.Ontology
+    versionInfo = predicate(OWL.versionInfo)
+    imports = predicate(OWL.imports)
 
 def AllClasses(graph):
     prevClasses=set()
@@ -654,25 +815,25 @@ class Class(AnnotatibleTerms):
     The Manchester Syntax (supported in Protege) is used as the basis for the form 
     of this class
     
-    See: http://owl-workshop.man.ac.uk/acceptedLong/submission_9.pdf:
+    See: http://owl-workshop.man.ac.uk/acceptedLong/submission_9.pdf::
     
-    ‚ÄòClass:‚Äô classID {Annotation
-                  ( (‚ÄòSubClassOf:‚Äô ClassExpression)
-                  | (‚ÄòEquivalentTo‚Äô ClassExpression)
-                  | (‚ÄôDisjointWith‚Äô ClassExpression)) }
+        Class: classID {Annotation
+            ( (SubClassOf: ClassExpression)
+              | (EquivalentTo ClassExpression)
+              | (DisjointWith ClassExpression)) }
     
     Appropriate excerpts from OWL Reference:
     
-    ".. Subclass axioms provide us with partial definitions: they represent 
-     necessary but not sufficient conditions for establishing class 
-     membership of an individual."
+        ".. Subclass axioms provide us with partial definitions: they represent 
+        necessary but not sufficient conditions for establishing class 
+        membership of an individual."
      
-   ".. A class axiom may contain (multiple) owl:equivalentClass statements"   
-                  
-    "..A class axiom may also contain (multiple) owl:disjointWith statements.."
+        ".. A class axiom may contain (multiple) owl:equivalentClass statements"   
+        
+        "..A class axiom may also contain (multiple) owl:disjointWith statements.."
     
-    "..An owl:complementOf property links a class to precisely one class 
-      description."
+        "..An owl:complementOf property links a class to precisely one class 
+        description."
       
     """
     def _serialize(self,graph):
@@ -792,74 +953,43 @@ class Class(AnnotatibleTerms):
         True
         """
         return BooleanClass(operator=OWL.intersectionOf,members=[self,other],graph=self.graph)
-            
-    def _get_subClassOf(self):
-        for anc in self.graph.objects(subject=self.identifier,predicate=RDFS.subClassOf):
-            yield Class(anc,
-                        graph=self.graph,
-                        skipOWLClassMembership=True)
-    def _set_subClassOf(self, other):
-        if not other:
-            return        
-        for sc in other:
-            self.graph.add((self.identifier,RDFS.subClassOf,classOrIdentifier(sc)))
-            
-    @TermDeletionHelper(RDFS.subClassOf)
-    def _del_subClassOf(self):            
-        pass            
-            
-    subClassOf = property(_get_subClassOf, _set_subClassOf, _del_subClassOf)
 
-    def _get_equivalentClass(self):
-        for ec in self.graph.objects(subject=self.identifier,predicate=OWL.equivalentClass):
-            yield Class(ec,graph=self.graph)
-    def _set_equivalentClass(self, other):
-        if not other:
-            return
-        for sc in other:
-            self.graph.add((self.identifier,OWL.equivalentClass,classOrIdentifier(sc)))
-            
-    @TermDeletionHelper(OWL.equivalentClass)
-    def _del_equivalentClass(self):            
-        pass                        
-            
-    equivalentClass = property(_get_equivalentClass, _set_equivalentClass, _del_equivalentClass)
+    class _subClassOf(object_predicate):
+        def __init__(self):
+            class _xClass(Individual):
+                def __new__(self, *av, **kw):
+                    kwa=kw.copy()
+                    kwa["skipOWLClassMembership"] = True
+                    return Class(*av, **kwa)
+            super(self.__class__, self).__init__(RDFS.subClassOf, _xClass)
+    subClassOf = _subClassOf()
 
-    def _get_disjointWith(self):
-        for dc in self.graph.objects(subject=self.identifier,predicate=OWL.disjointWith):
-            yield Class(dc,graph=self.graph)
-    def _set_disjointWith(self, other):
-        if not other:
-            return
-        for c in other:
-            self.graph.add((self.identifier,OWL.disjointWith,classOrIdentifier(c)))
-            
-    @TermDeletionHelper(OWL.disjointWith)
-    def _del_disjointWith(self):            
-        pass            
-                        
-    disjointWith = property(_get_disjointWith, _set_disjointWith, _del_disjointWith)
+    class _classPredicate(object_predicate):
+        def __init__(self, term):
+            class _xClass(Individual):
+                def __new__(self, *av, **kw):
+                    return Class(*av, **kw)
+            super(self.__class__, self).__init__(term, _xClass)
+
+    equivalentClass = _classPredicate(OWL.equivalentClass)
+    disjointWith = _classPredicate(OWL.disjointWith)
+
+    class _complement(object_predicate):
+        def __init__(self):
+            class _xClass(Individual):
+                def __new__(self, *av, **kw):
+                    return Class(*av, **kw)
+            super(self.__class__, self).__init__(OWL.complementOf, _xClass)
+        def _get(self, obj):
+            comp = list(super(self.__class__, self)._get(obj))
+            if not comp:
+                return None
+            elif len(comp) == 1:
+                return Class(comp[0],graph=self.graph)
+            else:
+                raise Exception(len(comp))
+    complementOf = _complement()
 
-    def _get_complementOf(self):
-        comp = list(self.graph.objects(subject=self.identifier,predicate=OWL.complementOf))
-        if not comp:
-            return None
-        elif len(comp) == 1:
-            return Class(comp[0],graph=self.graph)
-        else:
-            raise Exception(len(comp))
-        
-    def _set_complementOf(self, other):
-        if not other:
-            return
-        self.graph.add((self.identifier,OWL.complementOf,classOrIdentifier(other)))
-        
-    @TermDeletionHelper(OWL.complementOf)
-    def _del_complementOf(self):            
-        pass            
-                
-    complementOf = property(_get_complementOf, _set_complementOf, _del_complementOf)
-    
     def isPrimitive(self):
         if (self.identifier,RDF.type,OWL.Restriction) in self.graph:
             return False
@@ -1035,7 +1165,12 @@ class EnumeratedClass(OWLRDFListProxy,Class):
     <BLANKLINE>
      ex:ogbujicBros a owl:Class;
          owl:oneOf ( ex:chime ex:uche ex:ejike ). 
-        
+    <BLANKLINE>
+     ex:chime a owl:Class. 
+    <BLANKLINE>
+     ex:ejike a owl:Class. 
+    <BLANKLINE>
+     ex:uche a owl:Class. 
     """
     _operator = OWL.oneOf
     def isPrimitive(self):
@@ -1440,15 +1575,17 @@ PropertyAbstractSyntax=\
 
 class Property(AnnotatibleTerms):
     """
-    axiom ::= 'DatatypeProperty(' datavaluedPropertyID ['Deprecated'] { annotation } 
-                { 'super(' datavaluedPropertyID ')'} ['Functional']
-                { 'domain(' description ')' } { 'range(' dataRange ')' } ')'
-            | 'ObjectProperty(' individualvaluedPropertyID ['Deprecated'] { annotation } 
-                { 'super(' individualvaluedPropertyID ')' }
-                [ 'inverseOf(' individualvaluedPropertyID ')' ] [ 'Symmetric' ] 
-                [ 'Functional' | 'InverseFunctional' | 'Functional' 'InverseFunctional' |
-                  'Transitive' ]
-                { 'domain(' description ')' } { 'range(' description ')' } ')    
+    What is a property?::
+
+        axiom ::= 'DatatypeProperty(' datavaluedPropertyID ['Deprecated'] { annotation } 
+                    { 'super(' datavaluedPropertyID ')'} ['Functional']
+                    { 'domain(' description ')' } { 'range(' dataRange ')' } ')'
+                | 'ObjectProperty(' individualvaluedPropertyID ['Deprecated'] { annotation } 
+                    { 'super(' individualvaluedPropertyID ')' }
+                    [ 'inverseOf(' individualvaluedPropertyID ')' ] [ 'Symmetric' ] 
+                    [ 'Functional' | 'InverseFunctional' | 'Functional' 'InverseFunctional' |
+                      'Transitive' ]
+                    { 'domain(' description ')' } { 'range(' description ')' } ')    
     """
     def __init__(self,identifier=None,graph = None,baseType=OWL.ObjectProperty,
                       subPropertyOf=None,domain=None,range=None,inverseOf=None,
@@ -1599,6 +1736,7 @@ class Property(AnnotatibleTerms):
         pass            
                                 
     range = property(_get_range, _set_range, _del_range)
+
         
 def test():
     import doctest
